#!/bin/bash
# Generic buildstep script

if [ -r "$PLATFORM_LOC/scripts/utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/utils.sh"
else
	source "$script_lib_dir/utils.sh"
fi

do_setup_cmake_options() {
	# Native and cross packages must be configured to execute in the
	# platform-provided native sysroot
	local hbase # main base directory, files that must be on the host
	local tbase # target base, for files that should be on the target
	# --prefix should be on the target (tbase)
	#	except that --bindir, --sbindir and --libexec MUST be on host (hbase)
	# --exec_prefix (so rootdatadir and sysconfdir) should be on target (tbase)
	local opt val var base
	
	# Choose the CMake generator (ensure these are first in cmake options to be nicer)
	if [ -z "$cmake_generator" -o "$cmake_generator" = "Unix Makefiles" ]; then
		cmake_generator='Unix Makefiles'
		cmake_make_program=make
	elif [ "$cmake_generator" = "Ninja" ]; then
		cmake_generator_name="Ninja"
		cmake_make_program=ninja
	else
		warning "Unsupported CMake generator '$cmake_generator'"
		warning "Reverting to 'Unix Makefiles'."
		cmake_generator='Unix Makefiles'
		cmake_make_program=make
	fi
	cmake_options+=( -G "$cmake_generator" "$(requote_arg -DCMAKE_MAKE_PROGRAM=$cmake_make_program)" )
	
	if [ "$DO_NOT_UPDATE_INSTALL_DIRECTORIES" != "true" ]; then
		if $recipe_is_native; then
			hbase="${SYSROOT_LOC%/}"
			tbase="${hbase}"
			if $recipe_is_cross; then
				tbase="${TARGET_SYSROOT_LOC%/}"
			fi
		else
			hbase=""
			tbase=""
		fi
	else
		hbase=""
		tbase=""
	fi
	
	while [ $# -gt 0 ]; do
		opt="$1"
		shift
		case $opt in
			-D*=*)
				val=${opt#-D*=}
#				if [ -z "$val" ]; then continue; fi
				opt=${opt#-D}
				opt=${opt%%=*}
				;;
			-D)
				if [ $# -eq 0 ]; then
					fatal 1 "Incorrect -D option without a value"
				fi
				opt="$1"
				shift
				val=${opt#*=}
#				if [ -z "$val" ]; then continue; fi
				opt=${opt%%=*}
				;;
			-D*)
				val=
				opt=${opt#-D}
				opt=${opt%%=*}
				;;
				
			*)
				cmake_options+=( "$(requote_arg $opt)" )
				continue
				;;
		esac
				
		base=""
		var="${opt,,}"
		if [ "$opt" = "CMAKE_INSTALL_PREFIX" ]; then
			if ! [[ " $val" =~ " $tbase/" ]]; then
				case $val in
					TARGET/*)
						base="$tbase"
						val=${val#TARGET}
						;;
					NATIVE/*)
						base="$hbase"
						val=${val#NATIVE}
						;;
					*)
						base="$tbase"
						;;
				esac
			fi
		fi
		if [ -n "$base" ]; then
			val=${val#/}
			val="$base/$val"
		fi

		eval $var="$(requote_arg $val)"
		cmake_options+=( "-D$opt=${!var}" )
	done

	# Provide sensible defaults for most common variables
	set_cmake_default CMAKE_INSTALL_PREFIX "$prefix"
	set_cmake_default CMAKE_INSTALL_BINDIR "${bindir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_SBINDIR "${sbindir#${prefix}/}"
    	set_cmake_default CMAKE_INSTALL_LIBEXECDIR "${libexecdir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_SYSCONFDIR "${sysconfdir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_SHAREDSTATEDIR "${sharedstatedir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_LOCALSTATEDIR "${localstatedir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_RUNSTATEDIR "${runstatedir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_LIBDIR "${libdir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_INCLUDEDIR "${includedir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_OLDINCLUDEDIR "${oldincludedir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_DATAROOTDIR "${datarootdir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_DATADIR "${datadir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_INFODIR "${infodir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_LOCALEDIR "${localedir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_MANDIR "${mandir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_DOCDIR "${docdir#${prefix}/}"
	set_cmake_default CMAKE_INSTALL_SO_NOEXE "0"

	if $recipe_is_native; then
		set_cmake_default CMAKE_SYSTEM_PROCESSOR "${configure_build_alias%%-*}"
		# Not sure setting CMAKE_CROSSCOMPILING is needed, should be set automatically
		# set_cmake_default CMAKE_CROSSCOMPILING "false"
	else
		set_cmake_default CMAKE_SYSTEM_NAME "Linux"
		local arch="$arch_name"
		if [ "$arch" = "powerpc" ]; then arch="ppc"
		elif [ "$arch" = "powerpc64" ]; then arch="ppc64"
		fi
		set_cmake_default CMAKE_SYSTEM_PROCESSOR "$arch"
		# Not sure setting CMAKE_CROSSCOMPILING is needed, should be set automatically
		# set_cmake_default "CMAKE_CROSSCOMPILING" "true"
	fi
    	set_cmake_default CMAKE_VERBOSE_MAKEFILE "1"
    	set_cmake_default CMAKE_NO_SYSTEM_FROM_IMPORTED "1"
	set_cmake_default CMAKE_SOURCE_DIR "$srcDir"
	set_cmake_default CMAKE_BINARY_DIR "$buildDir"
	set_cmake_default CMAKE_TOOLCHAIN_FILE "$workDir/toolchain.cmake"
	
	# Generate the toolchain file (This may deserve a separate task to be ap/pre-pendable)
	generate_cmake_toolchain_file
	
	# Apply the generated defaults
	original_cmake_flags="$cmake_flags"
	if $recipe_is_native; then
		cmake_options+=( "-Wno-dev" )
	fi
	cmake_flags="${cmake_options[*]}" # You should rather not use this, use $cmake_options
}		


do_buildstep_prepend

do_buildstep

do_buildstep_append